/*
 * Copyright (c) 1991-1993 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the Computer Systems
 *	Engineering Group at Lawrence Berkeley Laboratory.
 * 4. Neither the name of the University nor of the Laboratory may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static const char rcsid[] =
    "@(#) $Header: audio-ums.cc,v 1.6 96/02/04 23:57:24 mccanne Exp $ (LBL)";
#endif

#include <string.h>

#include <UMSAudioDevice.xh>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <sys/select.h>


#include "audio.h"
#include "Tcl.h"

#define ULAW_ZERO 0x7f
#define ABUFLOG2 8
#define ABUFLEN (1 << ABUFLOG2)
#define NFRAG 5

#define RESULT printf("Call at %s:%d returned rc=%d\n", __FILE__, __LINE__, rc);


class UMSAudio : public Audio {
public:
    UMSAudio();
    virtual int FrameReady();
    virtual u_char* Read();
    virtual	void Write(u_char *);
    virtual void SetRGain(int);
    virtual void SetPGain(int);
    virtual void OutputPort(int);
    virtual void InputPort(int);
    virtual void Obtain();
    virtual void Release();
    virtual void RMute();
    virtual void RUnmute();

protected:

    UMSAudioDevice *record_device, *play_device;

    UMSAudioTypes_Buffer record_buffer, play_buffer;

    Environment *ev;

    char *in_name, *out_name;

    void setup_input();
    void setup_output();

    FILE *out_fp;
};

static class UMSAudioMatcher : public Matcher {
public:
	UMSAudioMatcher() : Matcher("audio") {}
	TclObject* match(const char* fmt) {
		if (strcmp(fmt, "ums") == 0)
			return (new UMSAudio);
		else
			return (0);
	}
} UMS_audio_matcher;

UMSAudio::UMSAudio()
{
    ev = somGetGlobalEnvironment();

}

void UMSAudio::Obtain()
{
    UMSAudioDeviceMClass *audio_device_class;
    UMSAudioDeviceMClass_ErrorCode audio_device_class_error;
    char* error_string;
    char* audio_formats_alias;
    char* audio_inputs_alias;
    char* audio_outputs_alias;
    long r;
    int rc;

    long left_gain, right_gain;
    
    
    if (HaveAudio())
	abort();

    out_fp = fopen("/tmp/out.au", "w");

    printf("Obtain()\n");

#if 1
    audio_device_class = new UMSAudioDeviceMClass();
    record_device = audio_device_class->make_by_alias(ev,
					     "Audio",
					     "RECORD",
					     UMSAudioDevice_BlockingIO,
					     &audio_device_class_error,
					     &error_string,
					     &audio_formats_alias,
					     &audio_inputs_alias,
					     &audio_outputs_alias);
#else
    record_device = new UMSAudioDevice();
    rc = record_device->open(ev, "/dev/baud0/1",
			     "RECORD", UMSAudioDevice_BlockingIO);
    RESULT;
#endif

    if (record_device == NULL)
    {
	fprintf(stderr, "can't create recording device\n");
	exit(1);
    }

#if 1
    printf("Create record succeeds, formats=%s inputs=%s outputs=%s\n",
	   audio_formats_alias, audio_inputs_alias, audio_outputs_alias);
#endif
	
#if 1
    play_device = audio_device_class->make_by_alias(ev,
					     "Audio",
					     "PLAY",
					     UMSAudioDevice_BlockingIO,
					     &audio_device_class_error,
					     &error_string,
					     &audio_formats_alias,
					     &audio_inputs_alias,
					     &audio_outputs_alias);
    if (play_device == NULL)
    {
	fprintf(stderr, "can't create recording device\n");
	exit(1);
    }

    printf("Create play succeeds, formats=%s inputs=%s outputs=%s\n",
	   audio_formats_alias, audio_inputs_alias, audio_outputs_alias);

#endif

    rc = record_device->set_sample_rate(ev, 8000, &r);
    RESULT;
    rc = record_device->set_bits_per_sample(ev, 8);
    RESULT;
    rc = record_device->set_number_of_channels(ev, 1);
    RESULT;
    rc = record_device->set_audio_format_type(ev, "PCM");
    RESULT;
    rc = record_device->set_byte_order(ev, "MSB");
    RESULT;
    rc = record_device->set_number_format(ev, "TWOS_COMPLEMENT");
    RESULT;

    rc = record_device->set_volume(ev, 100);
    RESULT;
    rc = record_device->set_balance(ev, 0);
    RESULT;
    rc = record_device->set_time_format(ev, UMSAudioTypes_Msecs);
    RESULT;

    rc = record_device->set_monitor(ev, 0);
    RESULT;

    left_gain = right_gain = 100;
    rc = record_device->enable_input(ev, "LINE_IN", &left_gain, &right_gain);
    RESULT;
    printf("gains: %d %d\n", left_gain, right_gain);

    rc = record_device->initialize(ev);
    RESULT;
	
#if 1
    rc = play_device->set_sample_rate(ev, 8000, &r);
    RESULT;
    rc = play_device->set_bits_per_sample(ev, 8);
    RESULT;
    rc = play_device->set_number_of_channels(ev, 1);
    RESULT;
    rc = play_device->set_audio_format_type(ev, "PCM");
    RESULT;
    rc = play_device->set_byte_order(ev, "MSB");
    RESULT;
    rc = play_device->set_number_format(ev, "TWOS_COMPLEMENT");
    RESULT;

    rc = play_device->set_volume(ev, 100);
    RESULT;
    rc = play_device->set_balance(ev, 0);
    RESULT;
    rc = play_device->set_time_format(ev, UMSAudioTypes_Msecs);
    RESULT;

    left_gain = right_gain = 100;
    rc = play_device->enable_output(ev, "LINE_OUT", &left_gain, &right_gain);
    RESULT;

    rc = play_device->initialize(ev);
    RESULT;
#endif
    
    iports = 2;
    oports = 1;

    record_buffer._length = 0;
    record_buffer._buffer = new octet[16384];
    record_buffer._maximum = 16384;

    rc = record_device->start(ev);
    RESULT;

    long mfd;

    rc = record_device->get_fd(ev, &mfd);
    RESULT;
    
    fd = mfd;

    Audio::Obtain();
}

void UMSAudio::Release()
{
    if (fd >= 0) {
	Audio::Release();
    }
}

void UMSAudio::Write(u_char *cp)
{
    if (HaveAudio() && (rmute & 1) != 0)
    {
	long n;
	play_buffer._buffer = cp;
	play_buffer._length = 160;
	play_buffer._maximum = 160;
	play_device->write(ev, &play_buffer, 160, &n);
	printf("wrote %d samples\n", n);
    }
}

int UMSAudio::FrameReady()
{
    int rc;
    long n;
    static double lt;
    double t,del;
    struct timeval tv;

//    gettimeofday(&tv, 0);
//    t = (double) tv.tv_sec + (double) tv.tv_usec / 1000000;
//    del = t - lt;
//    lt = t;
    
    rc = record_device->read(ev, &record_buffer, 160, &n);

    if (out_fp)
    {
	unsigned char *c;
	for (int i = 0; i < 160; i++)
	{
	    c = (unsigned char *) record_buffer._buffer[i];
	    fprintf(out_fp, "%02x ", c);
	    if ((i % 20) == 0)
		fprintf(out_fp, "\n");
	}

	fflush(out_fp);
    }
//    printf("read returns rc=%d n=%d delta=%f\n", rc, n, del);
    return 1;
}

u_char* UMSAudio::Read()
{
    u_char* cp;

/*
    printf("Read()\n");

    if (!HaveAudio() || ((rmute & 1) != 0))
	cp = zbuf;
    else {
	cp = ubuf;
	ubufptr = cp;
    }
    return (cp);
    */
    return record_buffer._buffer;
}

void UMSAudio::SetRGain(int level)
{
    printf("set rgain %d\n", level);

    rgain = level;
    
    setup_input();
}

void UMSAudio::SetPGain(int level)
{
    printf("set pgain %d\n", level);
    pgain = level;

    setup_output();
}

void UMSAudio::OutputPort(int p)
{
    printf("outputport %d\n", p);
    oport = p;

    switch (p)
    {
    case output_speaker:
	out_name = "LINE_OUT";
	break;
    }

    setup_output();
}

void UMSAudio::InputPort(int p)
{
    printf("inputport %d\n", p);
    iport = p;

    switch (p)
    {
    case input_mike:
	in_name = "HIGH_GAIN_MIKE";
	break;

    case input_line:
	in_name = "LINE_IN";
	break;
    }

    setup_input();
}

void UMSAudio::setup_input()
{
    long left_gain, right_gain, vol;
    int rc;

    vol = left_gain = right_gain = (long) ((double) rgain * 100.0 / 256.0);

    rc = record_device->stop(ev);
    RESULT;

    rc = record_device->enable_input(ev, in_name, &left_gain, &right_gain);
    RESULT;
    
    printf("Set record gains to %d %d input %s\n",
	   left_gain, right_gain, in_name);

    rc = record_device->initialize(ev);
    RESULT;

    rc = record_device->start(ev);
    RESULT;
}

void UMSAudio::setup_output()
{
    long left_gain, right_gain;
    int rc;

    left_gain = right_gain = (long) ((double) pgain * 100.0 / 256.0);

    rc = play_device->stop(ev);
    RESULT;

    rc = play_device->enable_output(ev, out_name, &left_gain, &right_gain);
    RESULT;
    
    printf("Set record gains to %d %d output %s\n",
	   left_gain, right_gain, out_name);

    rc = play_device->start(ev);
    RESULT;
    
}

void UMSAudio::RMute()
{
    printf("mute\n");
    rmute |= 1;
}

void UMSAudio::RUnmute()
{
    printf("unmute\n");
    rmute &=~ 1;
}

