/* To: czahl@cs.tu-berlin.de
 * Subject: audio.h
 * Date: Mon, 15 May 95 15:05:12 -0500
 * From: "Paul Wadehra" <paul@austin.ibm.com>
 *
 *
 * Here is the updated audio.h.
 *
 * ----------------
 */
#ifndef _H_AUDIO
#define _H_AUDIO

#include <sys/param.h>

/* These are the defined ioctl control operations */
#define AUDIO_INIT                 0   /* obtain adapter information */
#define AUDIO_STATUS               1   /* obtain current adapter status */
#define AUDIO_CONTROL              2   /* change adpater characteristics */
#define AUDIO_BUFFER               3   /* query buffer status */
#define AUDIO_LOAD                 4   /* load DSP code */
#define AUDIO_WAIT                 5   /* let kernel write buffers drain */
#define AUDIO_SET_CHANNELS         10  /* control hardware settings */
#define AUDIO_GET_CHANNELS         11  /* query hardware settings */
#define AUDIO_CHANNEL_STATUS       12  /* query current hardware state */
#ifdef ENABLE_NOTIFY
#define AUDIO_NOTIFY               13  /* request signal at event */
#endif /* ENABLE_NOTIFY */
#define AUDIO_SET_GAIN             14  /* change capture gain */
#define AUDIO_MODIFY_LIMITS        20  /* change settings for limits */
#define AUDIO_MASTER_VOLUME        21  /* master volume in percentage */

#define LOAD_PATH       MAXPATHLEN      /* max # of chars in DSP path */

/* This is the definition for information about a sampling rate. */
typedef struct _audio_init
{
  long srate;                           /* the sampling rate in Hz */
  long bits_per_sample;                 /* the number of bits per sample */
  long bsize;                           /* block size for this rate */
  int mode;                             /* PCM, ADPCM, ADPCMS, etc. */
  int channels;                         /* number of audio channels */
  long position_resolution;             /* smallest increment for position */
  char loadpath[LOAD_PATH];             /* path of DSP code to be loaded */
  unsigned long flags;                  /* variable, fixed, etc. */
  unsigned long operation;              /* the desired operation */
  int rc;                               /* return code for requested op */
  int slot_number;                      /* slot number of the adapter */
  int device_id;                        /* adapter identification number */
  void *reserved;                       /* reserved pointer */
} audio_init;

/* These are the defined values for audio_query.mode. */
#define ADPCM           1
#define PCM             2
#define MU_LAW          3               /* mu-law */
#define A_LAW           5
#define SOURCE_MIX      6
#define IMA_ADPCM       13
#define ANALOG_MIX      50

/* The following defines can be used as input information in */
/* audio_init.flags. */
#define FIXED                   ( 1 << 0 )	/* fixed length data */
#define LEFT_ALIGNED            ( 1 << 1 )      /* variable length data */
#define RIGHT_ALIGNED           ( 1 << 2 )      /* variable length data */
#define TWOS_COMPLEMENT		( 1 << 3 )	/* 2's complement data */
#define SIGNED                  ( 1 << 4 )      /* unsigned data */
#define BIG_ENDIAN              ( 1 << 5 )      /* high order bit is to */
						/* the left [MSB] */

/* The following flags are returned by AUDIO_INIT as output information */
/* in audio_init.flags. */
#define PITCH                   ( 1 << 20 )     /* picth is supported */
#define INPUT                   ( 1 << 21 )     /* input is supported */
#define OUTPUT                  ( 1 << 22 )     /* output is supported */
#define MONITOR                 ( 1 << 23 )     /* monitor is supported */
#define VOLUME                  ( 1 << 24 )     /* volume is supported */
#define VOLUME_DELAY            ( 1 << 25 )     /* volume delay is supported */
#define BALANCE                 ( 1 << 26 )     /* balance is supported */
#define BALANCE_DELAY           ( 1 << 27 )     /* balance delay is supported */
#define TREBLE                  ( 1 << 28 )     /* treble control is supported */
#define BASS                    ( 1 << 29 )     /* bass control is supported */
#define BESTFIT_PROVIDED        ( 1 << 30 )	/* best fit returned */
#define LOAD_CODE               ( 1 << 31 )	/* DSP load needed */

/* These are the defined values for audio.init.operation. */
#define PLAY                    1
#define RECORD                  2

/* These are the defined values for audio_init.rc. */
#define NO_PLAY                 1       /* DSP code can't do play requests */
#define NO_RECORD               2       /* DSP code can't do record requests */
#define INVALID_REQUEST         4       /* request was invalid */
#define CONFLICT                5       /* conflict with open's flags */
#define OVERLOADED		6	/* out of DSP MIPS or memory */

#define AUDIO_IGNORE            -1      /* ignore this field */

/* These are the defined values for audio_init.device_id. */
#define MACPA                   2       /* this is the supported adapter */
#define BASE_4231               0x100   /* CS4231 codec on the planar */
#define MCA_4231                0x101   /* CS4231 codec on an adapter */

#define INIT_BUFFER_REQUEST     100     /* request for additional */
					/* device initialization */

typedef struct _init_buf_req
{
  long buffer_size;                     /* user-specified size of kernel */
					/* queue for this operation */
  long play_block_count;                /* # of blocks to accumulate */
					/* before starting playback */
  long play_lower_limit;                /* unblock select only when this */
					/* number of bytes remains in the */
					/* playback kernel buffer */
  long cap_upper_limit;                 /* unblock select when this number */
					/* of bytes is in the capture */
					/* kernel buffer */
  long byte_count;                      /* unblock select when this number of */
                                        /* bytes has been processed */
  long return_code;                     /* 0 if no error; non-zero otherwise */
} init_buf_req;

/* This is used when changing the adapter state. */
typedef struct _audio_change
{
  void *dev_info;                       /* ptr to device dependent info */
  long input;                           /* the new input source */
  long output;                          /* the new output */
  long monitor;                         /* the new monitor state */
  long volume;                          /* the new volume level */
  long volume_delay;                    /* the new volume delay */
  long balance;                         /* the new balance */
  long balance_delay;                   /* the new balance delay */
  long treble;                          /* the new treble state */
  long bass;                            /* the new bass state */
  long pitch;                           /* the new pitch state */
} audio_change;

#ifndef TRUE
#define TRUE    1
#endif
#ifndef FALSE
#define FALSE   0
#endif

/* These are the values defined for audio_change.input. */
#define HIGH_GAIN_MIKE          0
#define LINE_1                  1
#define LINE_2                  2
#define LINES_1AND2             3
#define LOW_GAIN_MIKE           4
#define AUX1                    100
#define AUX2                    101
#define AUX1_LEFT               206
#define AUX1_RIGHT              207
#define AUX2_LEFT               208
#define AUX2_RIGHT              209
#define RECORD_MONITOR          250
#define ALL_LINES               0xFFFFFFF

/* These are the values defined for audio_change.output. */
#define EXTERNAL_SPEAKER        1       /* external speaker */
#define INTERNAL_SPEAKER        2       /* internal speaker */
#define OUTPUT_1                4       /* first output */
#define INTERNAL_SPEAKER_ON     10      /* turn on codec to internal spkr */
#define INTERNAL_SPEAKER_OFF    11      /* turn off codec to internal spkr */
#define INTERNAL_AUDIO_ON       12      /* turn on all internal audio */
#define INTERNAL_AUDIO_OFF      13      /* turn off all internal audio */

/* These are the values defined for audio_change.monitor. */
#define MONITOR_OFF             0       /* no monitoring */
#define MONITOR_UNCOMPRESSED    1       /* uncompressed monitoring */
#define MONITOR_COMPRESSED      2       /* compressed monitoring */

/* This is the definition for information about the adapter's current state. */
typedef struct _audio_status
{
  long srate;                           /* the sampling rate in Hz */
  long bits_per_sample;                 /* the number of bits per sample */
  long bsize;                           /* block size for this rate */
  int mode;                             /* PCM, ADPCM, MU_LAW, etc. */
  int channels;                         /* number of audio channels */
  unsigned long flags;                  /* various flags */
  unsigned long operation;              /* the current operation in progress */
  audio_change change;                  /* status of input, output, etc. */
} audio_status;

/* These are the values defined for operation. */
#define STOPPED                 0       /* no operation is in progress */
#define PLAYING                 1       /* playback mode is on */
#define RECORDING               2       /* record mode is on */
#define UNINITIALIZED           0xFFFFFFFF     /* adapter is uninitialized */

/* This is the definition that defines an AUDIO_CONTROL request. */
typedef struct _audio_control
{
  unsigned int ioctl_request;           /* the desired ioctl request */
  void *request_info;                   /* the request specifics */
  unsigned long position;               /* # of units before executing it */
  int return_code;                      /* contains any error code */
} audio_control;

/* These are the values defined for audio_control.ioctl_request. */
#define AUDIO_CHANGE    0               /* change adapter characteristics */
#define AUDIO_START     1               /* start new operation */
#define AUDIO_STOP      2               /* stop current operation */
#define AUDIO_PAUSE     3               /* suspend the current operation */
#define AUDIO_RESUME    4               /* resume a suspended operation */

/* This is the definition for information about the buffer state. */
typedef struct _audio_buffer
{
  unsigned long flags;                  /* indicates whether error occurred */
  unsigned long read_buf_size;          /* # of bytes in read queue */
  unsigned long write_buf_size;         /* # of bytes in write queue */
  unsigned long read_buf_time;          /* # of milliseconds in read queue */
  unsigned long write_buf_time;         /* # of milliseconds in write queue */
  unsigned long read_buf_max;           /* max # of bytes ever in read queue */
  unsigned long write_buf_max;          /* max # of bytes ever in write queue */
  unsigned long position;               /* # of milliseconds since last START */
  unsigned long position_type;          /* type of unit associated with */
					/* the position field */
  long read_buf_cap;                    /* max capacity of read queue */
  long write_buf_cap;                   /* max capacity of write queue */
  long request_buf_cap;                 /* max number of requests that can be */
					/* enqueued */
} audio_buffer;

/* These are the possible error states that can be flagged in */
/* audio_buffer.flags. */
#define AUDIO_UNDERRUN  ( 1 << 0 )      /* indicates underflow occurred */
#define AUDIO_OVERRUN   ( 1 << 1 )      /* indicates overflow occurred */
#define FIFO_UNDERRUN   ( 1 << 2 )      /* indicates underflow occurred */
#define FIFO_OVERRUN    ( 1 << 3 )      /* indicates overflow occurred */

/* These are the possible meanings for audio_buffer.position_type. */
#define POS_MSECS       0               /* position value is milliseconds */

/* This is the structure used in conjunction with the AUDIO_LOAD command. */
/* When used, the application loads the file specified by the device */
/* driver into the memory buffer described by the audio_load structure. */
typedef struct _audio_load
{
  char *buffer;                         /* ptr to buffer where code has */
					/* been loaded */
  unsigned long size;                   /* number of bytes in buffer */
  unsigned long flags;                  /* contains flags about buffer */
} audio_load;

#define LOAD_START      1               /* 1st part of buffer to load */
#define LOAD_END        2               /* last part of buffer to load */

/* Number of channel types */
#define MAX_TABLE_SIZE        30

/* Mute flags */
#define INPUT_MUTE_OFF        0
#define INPUT_MUTE_ON         1
#define OUTPUT_MUTE_OFF       2
#define OUTPUT_MUTE_ON        3

/* Structures used in conjunction with the AUDIO_SET_CHANNELS ioctl. */
typedef struct _channel_setting
{
  unsigned long channel_type;           /* channel specification */
  unsigned long channel;                /* left, right channel */
  unsigned long channel_gain;           /* gain or attenuation level */
  unsigned long gain_delay;             /* milliseconds delay */
  unsigned long mute;                   /* mute on or off */
} channel_setting;

typedef struct _channels_list
{
  long size;                            /* number of elements */
  channel_setting *csptr;               /* address of first */
					/* channel_setting element */
} channels_list;

/* Structure used in conjunction with the AUDIO_GET_CHANNELS ioctl. */
typedef struct _audio_get_channels
{
  unsigned long device_owner;              /* ID of the device owner */
  int device_id;                           /* device id */
  unsigned long driver_version;            /* driver software version number */
  unsigned long channel_count;             /* number of initialized entries */
					   /* in channel_type[] upon return */
  unsigned long channel_type[MAX_TABLE_SIZE]; /* channel specification */
  unsigned long support[MAX_TABLE_SIZE];   /* bit field of the hardware functions */
					   /* available on a channel */
  unsigned long rate_count;                /* number of initialized entries */
					   /* in rates[] upon return */
  unsigned long rates[MAX_TABLE_SIZE];     /* table of supported sampling rates */
  unsigned long format_count;              /* number of initialized entries in */
					   /* formats[] upon return */
  unsigned long formats[MAX_TABLE_SIZE];   /* table of supported data formats */
  int return_code;                         /* specific error return code */
  void *reserved;
} audio_get_channels;

/* audio_get_channels.device_id values */
#define BASE_4231             0x100
#define ADAPTER_4231          0x101

/* valid audio_get_channels.channel_type values */
#define HIGH_GAIN_MIC         0
#define LOW_GAIN_MIC          1
#define LINE1_IN              10
#define MONO_IN               50
#define AUX1_IN               60
#define AUX2_IN               61
#define HOST_INPUT            90
#define LINE1_OUT             100
#define MONO_OUT              140
#define HOST_OUTPUT           160

/* valid audio_get_channels.support values */
#define LEFT_CHANNEL          (1 << 0)         /* left channel is available */
#define RIGHT_CHANNEL         (1 << 1)         /* right channel is available */
#define MUTE                  (1 << 2)         /* mute can be set */
#define GAIN                  (1 << 3)         /* gain level can be set */
#define GAIN_DELAY            (1 << 4)         /* gain delay can be set */

typedef struct _audio_channel_status
{
  unsigned long channel_count;  /* number of initialized entries */
				/* in the table of inputs and outputs. */
  unsigned long channel_type[MAX_TABLE_SIZE]; /* channel specification */
  unsigned long channel[MAX_TABLE_SIZE];      /* left or right channel */
  unsigned long gain[MAX_TABLE_SIZE];         /* gain setting */
  unsigned long gain_delay[MAX_TABLE_SIZE];   /* gain delay setting */
  unsigned long mute[MAX_TABLE_SIZE];         /* mute on or off setting */
  long master_volume;                         /* setting for master volume */
  long return_code;                           /* error return code */
  void *reserved;
} audio_channel_status;

/* The following defines are used in conjunction with poll(). */
/* They are in addition to POLLIN, POLLOUT, and POLLPRI. */
#define POLL_AUDIO_DATA_MOVEMENT      0x0100    /* data movement has occurred */
#define POLL_AUDIO_BUFFER_LIMIT       0x0200    /* buffer limit reached */
#define POLL_AUDIO_INTERRUPT          0x0400    /* interrupt has occurred */

/* This structure is used with the AUDIO_SET_GAIN ioctl. */
typedef struct _audio_set_gain
{
  signed long left_gain;         /* gain setting for left channel */
  signed long right_gain;        /* gain setting for right channel */
} audio_set_gain;

#endif /* _H_AUDIO */

